{-# language DataKinds #-}

module Fedi.Requests where

import Data.Aeson qualified as A
import Fedi.UserDetails
import Fedi.Crypto
import Network.HTTP.Req qualified as Req
import Data.ByteString.Lazy qualified as BSL

sendPost
  :: (A.FromJSON output, A.ToJSON input)
  => UserDetails
  -> String
  -> input
  -> IO output
sendPost details url payload = do
  let encoded = BSL.toStrict $ A.encode payload
  signed <- sign details encoded

  Req.runReq Req.defaultHttpConfig do
    r <-
      Req.req
        Req.POST
        (Req.https $ fromString url)
        (Req.ReqBodyBs encoded)
        Req.jsonResponse
        ( Req.header "ContentType" "application/activity+json"
          <> Req.header "Digest" signed.signedDigest
          <> Req.header "Signature" signed.signedMessage
        )
    pure $ Req.responseBody r

sendGet :: (A.FromJSON a) => String -> IO a
sendGet url = do
  Req.runReq Req.defaultHttpConfig do
    r <-
      Req.req
        Req.GET
        (Req.https $ fromString url)
        Req.NoReqBody
        Req.jsonResponse
        ( Req.header "ContentType" "application/activity+json"
        )
    pure $ Req.responseBody r
