module Fedi.Routes.Helpers where

import Data.Aeson qualified as A
import Data.ByteString qualified as BS
import Data.ByteString.Lazy qualified as BSL
import Web.Twain qualified as Twain
import Control.Monad.Catch (throwM)

jsonLD :: BSL.ByteString -> Twain.Response
jsonLD =
  Twain.raw
    Twain.status200
    [(Twain.hContentType, "application/activity+json; charset=utf-8")]

checkContentTypeAccept :: Twain.Request -> Bool
checkContentTypeAccept request =
  case lookup Twain.hAccept request.requestHeaders of
    Just bs ->
      ("application/activity+json" `BS.isInfixOf` bs)
        || ( ("application/ld+json" `BS.isInfixOf` bs)
              && ("profile=\"https://www.w3.org/ns/activitystreams\"" `BS.isInfixOf` bs)
           )
    Nothing -> False

parseJson :: (A.FromJSON a) => BSL.ByteString -> Twain.ResponderM a
parseJson body = do
  case A.eitherDecode body of
    Left msg -> throwM $ Twain.HttpError Twain.status400 msg
    Right a -> pure a
